import typing
from sys import stderr

from je_load_density.utils.exception.exception_tags import get_bad_trigger_function, get_bad_trigger_method
from je_load_density.utils.exception.exceptions import CallbackExecutorException
from je_load_density.utils.generate_report.generate_html_report import generate_html, generate_html_report
from je_load_density.utils.generate_report.generate_json_report import generate_json, generate_json_report
from je_load_density.utils.generate_report.generate_xml_report import generate_xml, generate_xml_report
from je_load_density.wrapper.start_wrapper.start_test import start_test


class CallbackFunctionExecutor(object):

    def __init__(self):
        self.event_dict: dict = {
            "user_test": start_test,
            "LD_generate_html": generate_html,
            "LD_generate_html_report": generate_html_report,
            "LD_generate_json": generate_json,
            "LD_generate_json_report": generate_json_report,
            "LD_generate_xml": generate_xml,
            "LD_generate_xml_report": generate_xml_report,
        }

    def callback_function(
            self,
            trigger_function_name: str,
            callback_function: typing.Callable,
            callback_function_param: [dict, None] = None,
            callback_param_method: str = "kwargs",
            **kwargs
    ):
        """
        :param trigger_function_name: what function we want to trigger only accept function in event_dict
        :param callback_function: what function we want to callback
        :param callback_function_param: callback function's param only accept dict
        :param callback_param_method: what type param will use on callback function only accept kwargs and args
        :param kwargs: trigger_function's param
        :return:
        """
        try:
            if trigger_function_name not in self.event_dict.keys():
                raise CallbackExecutorException(get_bad_trigger_function)
            execute_return_value = self.event_dict.get(trigger_function_name)(**kwargs)
            if callback_function_param is not None:
                if callback_param_method not in ["kwargs", "args"]:
                    raise CallbackExecutorException(get_bad_trigger_method)
                if callback_param_method == "kwargs":
                    callback_function(**callback_function_param)
                else:
                    callback_function(*callback_function_param)
            else:
                callback_function()
            return execute_return_value
        except Exception as error:
            print(repr(error), file=stderr)


callback_executor = CallbackFunctionExecutor()



class ProxyHTTPUser(object):

    def __init__(self):
        self.user_detail_dict = None
        self.tasks = None

    def setting(self, user_detail_dict: dict, tasks: dict):
        self.user_detail_dict = user_detail_dict
        self.tasks = tasks



template_keyword_1: list = [
    ["LD_start_test", {
        "user_detail_dict": {
            "user": "fast_http_user"
        },
        "user_count": 50, "spawn_rate": 10, "test_time": 5,
        **{
            "tasks": {
                "get": {"request_url": "http://httpbin.org/get"},
                "post": {"request_url": "http://httpbin.org/post"}
            }
        }
    }]
]

template_keyword_2: list = [
    ["LD_start_test", {
        "user_detail_dict": {
            "user": "http_user"
        },
        "user_count": 50, "spawn_rate": 10, "test_time": 5,
        **{
            "tasks": {
                "get": {"request_url": "http://httpbin.org/get"},
                "post": {"request_url": "http://httpbin.org/post"}
            }
        }
    }]
]



from locust import HttpUser, task, between

from je_load_density.wrapper.proxy.proxy_user import locust_wrapper_proxy


def set_wrapper_http_user(user_detail_dict: dict, **kwargs):
    locust_wrapper_proxy.user_dict.get("http_user").setting(user_detail_dict, **kwargs)
    return HttpUserWrapper


class HttpUserWrapper(HttpUser):
    """
    locust http user use to test
    """
    host = "http://localhost"
    wait_time = between(0.1, 0.2)

    def __init__(self, environment):
        super().__init__(environment)
        self.method = {
            "get": self.client.get,
            "post": self.client.post,
            "put": self.client.put,
            "patch": self.client.patch,
            "delete": self.client.delete,
            "head": self.client.head,
            "options": self.client.options,
        }

    @task
    def test(self):
        for test_task_method, test_task_data in locust_wrapper_proxy.user_dict.get("http_user").tasks.items():
            self.method.get(str(test_task_method).lower())(test_task_data.get("request_url"))





import sys
from threading import Lock

from je_load_density.utils.exception.exceptions import LoadDensityHTMLException
from je_load_density.utils.exception.exception_tags import html_generate_no_data_tag
from je_load_density.utils.test_record.test_record_class import test_record_instance

_html_string_head = \
    """
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8"/>
        <title>Load Density Report</title>
        <style>
          
            body{
                font-size: 100%;
            }
    
            h1{
                font-size: 2em;
            }
    
            .main_table {
                margin: 0 auto;
                border-collapse: collapse;
                width: 75%;
                font-size: 1.5em;
            }
    
            .success_table_head {
                border: 3px solid #262626;
                background-color: aqua;
                font-family: "Times New Roman", sans-serif;
                text-align: center;
            }
    
            .failure_table_head {
                border: 3px solid #262626;
                background-color: #f84c5f;
                font-family: "Times New Roman", sans-serif;
                text-align: center;
            }
    
            .table_data_field_title {
                border: 3px solid #262626;
                padding: 0;
                margin: 0;
                background-color: #dedede;
                font-family: "Times New Roman", sans-serif;
                text-align: center;
                width: 25%;
            }
    
            .table_data_field_text {
                border: 3px solid #262626;
                padding: 0;
                margin: 0;
                background-color: #dedede;
                font-family: "Times New Roman", sans-serif;
                text-align: left;
                width: 75%;
            }
    
            .text {
                text-align: center;
                font-family: "Times New Roman", sans-serif;
            }
        </style>
    </head>
    <body>
    <h1 class="text">
        Test Report
    </h1>
    """.strip()

_html_string_bottom = \
    """
    </body>
    </html>
    """.strip()

_success_table = \
    r"""
   <table class="main_table">
    <thead>
    <tr>
        <th colspan="2" class="success_table_head">Test Report</th>
    </tr>
    </thead>
    <tbody>
    <tr>
        <td class="table_data_field_title">Method</td>
        <td class="table_data_field_text">{Method}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">test_url</td>
        <td class="table_data_field_text">{test_url}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">name</td>
        <td class="table_data_field_text">{name}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">status_code</td>
        <td class="table_data_field_text">{status_code}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">text</td>
        <td class="table_data_field_text">{text}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">content</td>
        <td class="table_data_field_text">{content}</td>
    </tr>
    <tr>
        <td class="table_data_field_title">headers</td>
        <td class="table_data_field_text">{headers}</td>
    </tr>
    </tbody>
    </table>
    <br>
    """.strip()

_failure_table = \
    r"""
    <table class="main_table">
        <thead>
        <tr>
            <th colspan="2" class="failure_table_head">Test Report</th>
        </tr>
        </thead>
        <tbody>
        <tr>
            <td class="table_data_field_title">http_method</td>
            <td class="table_data_field_text">{http_method}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">test_url</td>
            <td class="table_data_field_text">{test_url}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">name</td>
            <td class="table_data_field_text">{name}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">status_code</td>
            <td class="table_data_field_text">{status_code}</td>
        </tr>
        <tr>
            <td class="table_data_field_title">error</td>
            <td class="table_data_field_text">{error}</td>
        </tr>
        </tbody>
    </table>
    <br>
    """.strip()


def generate_html():
    """
    :return: success_test_dict, failure_test_dict
    """
    if len(test_record_instance.test_record_list) == 0 and len(test_record_instance.error_record_list) == 0:
        raise LoadDensityHTMLException(html_generate_no_data_tag)
    else:
        success_list = list()
        for record_data in test_record_instance.test_record_list:
            success_list.append(
                _success_table.format(
                    Method=record_data.get("Method"),
                    test_url=record_data.get("test_url"),
                    name=record_data.get("name"),
                    status_code=record_data.get("status_code"),
                    text=record_data.get("text"),
                    content=record_data.get("content"),
                    headers=record_data.get("headers"),
                )
            )
        failure_list = list()
        if len(test_record_instance.error_record_list) == 0:
            pass
        else:
            for record_data in test_record_instance.error_record_list:
                failure_list.append(
                    _failure_table.format(
                        http_method=record_data.get("Method"),
                        test_url=record_data.get("test_url"),
                        name=record_data.get("name"),
                        status_code=record_data.get("status_code"),
                        error=record_data.get("error"),
                    )
                )
    return success_list, failure_list


def generate_html_report(html_name: str = "default_name"):
    """
    format html_string and output html file
    :param html_name: save html file name
    :return: html_string
    """
    _lock = Lock()
    success_list, failure_list = generate_html()
    try:
        _lock.acquire()
        with open(html_name + ".html", "w+") as file_to_write:
            file_to_write.writelines(
                _html_string_head
            )
            for success in success_list:
                file_to_write.write(success)
            for failure in failure_list:
                file_to_write.write(failure)
            file_to_write.writelines(
                _html_string_bottom
            )
    except Exception as error:
        print(repr(error), file=sys.stderr)
    finally:
        _lock.release()
